#!/bin/bash
script="${0##*/}"
script="${script%.*}"

#yellow start 
	C14="\e[1;33m"
#blue start 
	C01="\e[0;34m"
#color end
	CE="\e[0m"
#red start
	C04="\e[1;31m"
#black start
	C00="\e[0;30m"
#dark gray start
	C08="\e[1;30m"
#light blue start
	C09="\e[1;34m"
#green start
	C02="\e[0;32m"
#light green start
	C10="\e[1;32m"
#cyan start
	C03="\e[0;36m"
#light cyan start
	C11="\e[1;36m"
#light red start
	C12="\e[0;31m"
#purple start
	C05="\e[0;35m"
#light purple start
	C13="\e[1;35m"
#brown start
	C06="\e[0;33m"
#light gray start
	C07="\e[0;37m"
#white 
	C15="\e[1;37m"
  
#setting frequent stings
	YNYES="("$C14"Y"$CE"/"$C14"n"$CE")("$C14"Enter"$CE"=yes)"
	YNNO="("$C14"y"$CE"/"$C14"N"$CE")("$C14"Enter"$CE"=no)"

VERSION=1.0.0
READAK="read -n 1"

radio=""
mmap=""
port=""

function presskey() {
  echo ""
  echo "Press a key to continue..."
  read -n 1
}

prompt_confirm_no() {
  while true; do
    echo -e -n "${1:-Continue?} ${YNNO}: "
    read -e -r -n 1 REPLY
    case $REPLY in
      [yY]) echo ; return 0 ;;
      [nN]) echo; echo "Going back..."; sleep 1; return 1 ;;
      $'\0A') echo; echo "Going back..."; sleep 1; return 1 ;;
      *) printf " \033[31m %s \n\033[0m" "invalid input"
    esac 
  done  
}

prompt_confirm_yes() {
  while true; do
    echo -e -n "${1:-Continue?} $YNYES: "
    read -e -r -n 1 REPLY
    case $REPLY in
      [yY]) echo ; return 0 ;;
      [nN]) echo; echo "Going back..."; sleep 1; return 1 ;;
      $'\0A') echo; echo "Going back..."; sleep 1; return 0 ;;
      *) printf " \033[31m %s \n\033[0m" "invalid input"
    esac 
  done  
}

function initvars() {
  # radio
  if [ -f ./radio ]; then 
    radio=$(cat ./radio)
  else
    radio=""
  fi
  # port
  if [ -f ./port ]; then 
    port=$(cat ./port)
  else
    port=""
  fi
}

function get_mmap() {
  #chirpc -r <radio> --serial=<port> --mmap=<file> --download-mmap
  clear
  if [ -z $radio ]; then
    echo "No Radio selected!"
    presskey
    return -1
  fi
  if [ -z $port ]; then
    echo "No Port selected!"
    presskey
    return -1
  fi
  tmp="./files/${radio}_$(date +"%Y%m%d-%H%M%S").mmap"
  echo "Memory will be saved in file:"
  echo "> $tmp"
  echo ""
  prompt_confirm_no || return -1
  
  chirpc -r $radio --serial=$port --mmap=$tmp --download-mmap
  echo ""
  presskey
  clear
}

function select_port() {
  clear
  read -p "Enter port [/dev/ttyUSB0]: " port
  port=${port:-/dev/ttyUSB0}
  echo $port > ./port
  clear
}

function fzz() {
  #"right,50%,:wrap"
  fzf --ansi --bind 'enter:execute(echo {1})+abort' --bind 'end:last' --bind 'home:first' --header $"Radio: ${radio} | Port: ${port} | MMAP: ${mmap}"
  }


function list_settings(){
  clear
  if [ -z $mmap ]; then 
    echo "No file selected."
    presskey
    return -1
  fi
  chirpc --mmap=${mmap} --list-settings | less
  clear
}

function view_channel(){
  clear
  if [ -z $mmap ]; then 
    echo "No file selected."
    presskey
    return -1
  fi
  
  read -p "Enter channel: " chan
  isinteger $chan || return -1
  chirpc --mmap=${mmap} --get-mem $chan | less
  clear
}


function list_channels(){
  clear
  if [ -z $mmap ]; then 
    echo "No file selected."
    presskey
    return -1
  fi
  chirpc --mmap=${mmap} --list-mem | less
  clear
}

function select_radio() {  
  radio=$(chirpc --list-radios | sed 1d | sed -e 's/^[ \t]*//' | fzf)
  if [ ! -z $radio ]; then
    echo $radio > ./radio
  else
    radio="None"
  fi
}

function select_file() {  
  mmap=$(ls ./files | fzf)
  if [ -z $mmap ]; then
    mmap=""
  fi
}

function isinteger(){
  case $1 in
    ''|*[!0-9]*) echo ""; echo "Invalid number."; sleep 3; return -1;;
    *) return 0;;
  esac
}

function isfile() {
  if [ -z $mmap ]; then 
    echo "No file selected."
    presskey
    return -1
  fi
  return 0
}

function clear_mem(){
  read -p "Enter channel: " chan
  isinteger $chan || return -1
  prompt_confirm_no "Clear channel $chan ?" || return -1
  chirpc --mmap=${mmap} --clear-mem $chan 
  presskey
}

function copy_mem() {
  read -p "Enter channel to copy from: " chanf
  isinteger $chanf || return -1
  read -p "Enter channel to copy to: " chant
  isinteger $chant || return -1
  if [ $chanf == $chant ]; then
    echo -e $C04"Source and destination channels are same. Aborting."$CE
    sleep 3
    return 1
  fi
  prompt_confirm_no "Copy channel $chanf to position $chant ?" || return -1
  
  chirpc --mmap=${mmap} --copy-mem $chanf $chant
}

function channel_operations() {
  isfile || return 1
  option=""
  while [ -z "$option"  ]
  do 
    option=$(echo "1. Clear Memory Channel
2. Copy Channel
X. Back" | fzz)

    if [ ! -z "$option" ]
    then
      ch="$option"
      clear
      case $ch in
        1.*) clear_mem;;
        2.*) copy_mem;;
        X.*) break;;
        esac
    fi
  option=""
  done
  clear
}


initvars
option=""
while [ -z "$option"  ]
do 
  option=$(echo "1. Select Radio
2. Select Port
3. Download from: $radio
4. Select MMAP File
5. List Settings of image file
6. List Channels
7. View Channel...
8. Channel Operations ->
F. File Manager (Midnight Commander)
X. Exit" | fzz)

  if [ ! -z "$option" ]
  then
    ch="$option"
    case $ch in
      1.*) select_radio;;
      2.*) select_port;;
      3.*) get_mmap;;
      4.*) select_file;;
      5.*) list_settings;;
      6.*) list_channels;;
      7.*) view_channel;;
      8.*) channel_operations;;
      F.*) mc;;
      X.*) exit;;
      esac
   
  # Cancel is pressed
  #else
  #        exec $maindir/main.sh
  fi
option=""
done

exit
